//---------------------------------------------------------------------
// Copyright (c) 2013 Galaxy Far East Corp, Inc. All rights reserved
// Galaxy Far East Corp
//---------------------------------------------------------------------
//              The Contrl and Status Registers with I2C Slave
//---------------------------------------------------------------------
// FILE NAME      : i2c_s.sv
// DEPARTMENT     : TS1, GFEC TW
// AUTHOR         : Slin
// AUTHORS EMAIL :
//---------------------------------------------------------------------
// RELEASE HISTORY
// VERSION DATE        AUTHOR  DESCRIPTION
// 1.0     2013-06-17  Slin     Created
// 1.1     2013-12-19  Slin     Modified for I/F changed.
// 2.0     2014-07-02  Slin     Adding a read done signal.
// 2.1     2014-07-22  SLin     Adding a clock stretching.
// 2.2     2014-07-22  SLin     Adding a soft reset for watchdog.
//---------------------------------------------------------------------
// PURPOSE : Short description of functionality
//---------------------------------------------------------------------
// PARAMETERS
//     PARAM NAME      RANGE    : DESCRIPTION           : DEFAULT
//    End_Of_ADDR      [7:0]    : End of Memory address : 0x1F
//---------------------------------------------------------------------
//---------------------------------------------------------------------
module i2c_s #(
    I2C_DevADDR = 7'h31)(
    input   logic         clk,
    input   logic         reset_n,
    input   logic         soft_reset_n,
    input   logic         scl_i,      // input SCL of I2C Bus.
    input   logic         sda_i,      // input SDA of I2C Bus.
    output  logic         sda_oe_n = 1'b1,
    input   logic  [7:0]  readdata,
    output  logic  [7:0]  writedata,
    output  logic  [7:0]  address,
    output  logic         read,
    output  logic         read_done,
    output  logic         write
    );

    // Local Parameters
    localparam  End_Of_ADDR = 8'hFF;

    // The Signals for SIPO Register
    logic       rxbuf_aclr = 1'b1;
    logic       rxbuf_clkenb;
    logic [7:0] rxbuf;

    // The Signals for PISO Register
    logic       txbuf_aclr = 1'b1;
    logic       txbuf_clkenb;
    logic       txbuf_load;
    logic       txbuf_shiftout;

    // The Counter Signals for SCL
    logic       sclcnt_aclr = 1'b1;
    logic       sclcnt_clkenb;
    logic [3:0] scl_cntnum;

    // The Signals for I2C Bus
    logic       scl_h2l_trans;
    logic       scl_l2h_trans;
    logic       scl_high;
    logic       sda_h2l_trans;
    logic       sda_l2h_trans;
    logic       sda_high;
    logic       scl_sync;
    logic       scl_d0 = 1'b1;
    logic       scl_d1 = 1'b1;
    logic       scl_d2 = 1'b1;
    logic       sda_sync;
    logic       sda_d0 = 1'b1;
    logic       sda_d1 = 1'b1;
    logic       sda_d2 = 1'b1;
    logic       i2c_sta;
    logic       i2c_sto;
    logic       i2c_op;
    logic       i2c_m_ack = 1'b1;
    logic       read_done_nxt;

    // The Signals for Misc.
    logic       reset_d1, reset_d2;
    logic       reset_sync;

    // The FSM is used to contrl I2C to CSRs
    enum int unsigned { STATE_1, STATE_2, STATE_3, STATE_4,
                        STATE_5, STATE_6, STATE_7, STATE_8,
                        STATE_9, STATE_10, STATE_11, STATE_12,
                        STATE_13, STATE_14, STATE_15, STATE_16,
                        STATE_17, STATE_18, STATE_19 } prev_state, curt_state;
    //-----------------------------------------------------------------
    // Regenerates a sync. reset signal.
    //-----------------------------------------------------------------

    wire reset_async_n = reset_n & soft_reset_n;

    always_ff @(posedge clk, negedge reset_async_n) begin
            if (!reset_async_n) begin
                reset_d2 <= 1'b0;
                reset_d1 <= 1'b0;
            end else begin
                reset_d2 <= reset_d1;
                reset_d1 <= 1'b1;
            end
        end

    assign reset_sync = ~reset_d2;

    //-----------------------------------------------------------------
    // Call a Resync. IO Module.
    //-----------------------------------------------------------------
    always_ff @(posedge clk) begin
        if (reset_sync) begin
            scl_d0           <= '1;
            scl_d1           <= '1;
            scl_d2           <= '1;
        end else begin
            scl_d0           <= scl_i;
            scl_d1           <= scl_d0;
            scl_d2           <= scl_d1;
        end
    end

    always_ff @(posedge clk) begin
        if (reset_sync) begin
            sda_d0 <= '1;
            sda_d1 <= '1;
            sda_d2 <= '1;
        end else begin
            sda_d0 <= sda_i;
            sda_d1 <= sda_d0;
            sda_d2 <= sda_d1;
        end
    end

    //-----------------------------------------------------------------
    // The SIPO Registers for I2C write operation.
    //-----------------------------------------------------------------
    sipo_8b RX_BUF (
        .aclr       (rxbuf_aclr),
        .clock      (clk),
        .enable     (rxbuf_clkenb),
        .shiftin    (sda_i),
        .q          (rxbuf)
        );

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                rxbuf_aclr = '1;
            else
                rxbuf_aclr <= ( (curt_state == STATE_1) ||
                                (curt_state == STATE_6) ||
                                (curt_state == STATE_9)
                               ) ;
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                rxbuf_clkenb <= '0;
            else
                rxbuf_clkenb <= ( ((curt_state == STATE_3) && scl_l2h_trans) ||
                                  ((curt_state == STATE_7) && scl_l2h_trans) ||
                                  ((curt_state == STATE_10) && scl_l2h_trans)
                                 );
        end

    //-----------------------------------------------------------------
    // The PISO Registers for I2C read operation.
    //-----------------------------------------------------------------
    piso_8b TX_BUF (
        .aclr       (txbuf_aclr),
        .clock      (clk),
        .data       (readdata),
        .enable     (txbuf_clkenb),
        .load       (txbuf_load),
        .shiftout   (txbuf_shiftout)
        );

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                txbuf_aclr <= '1;
            else
                txbuf_aclr <= ((curt_state == STATE_1) ||
                               (curt_state == STATE_13)) ;
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                txbuf_clkenb <= '0;
            else
                txbuf_clkenb <= ( //(curt_state == STATE_15) ||
                                  ((curt_state == STATE_14))||
                                  ((curt_state == STATE_16) && scl_h2l_trans)
                                 );
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                txbuf_load <= '0;
            else
                txbuf_load <= (curt_state == STATE_14);
        end

    //-----------------------------------------------------------------
    // The counter is used to conut amount of SCL.
    //-----------------------------------------------------------------
    cnt4b_up CNT_SCL (
        .aclr   (sclcnt_aclr),
        .clk_en (sclcnt_clkenb),
        .clock  (clk),
        .q      (scl_cntnum)
        );

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                sclcnt_aclr <= '1;
            else
                sclcnt_aclr <= ( (curt_state == STATE_1) ||
                                 (curt_state == STATE_6) ||
                                 (curt_state == STATE_9) ||
                                 (curt_state == STATE_13)
                                );
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                sclcnt_clkenb <= '0;
            else
                sclcnt_clkenb <= ( scl_h2l_trans &&
                                   ( (curt_state == STATE_3) ||
                                     (curt_state == STATE_5) ||
                                     (curt_state == STATE_7) ||
                                     (curt_state == STATE_8) ||
                                     (curt_state == STATE_10) ||
                                     (curt_state == STATE_11) ||
                                     (curt_state == STATE_16) ||
                                     (curt_state == STATE_17) )
                                   );
        end

    //-----------------------------------------------------------------
    // The Memory-Mapping Bus Statment
    //-----------------------------------------------------------------
    always_ff @(posedge clk)
        begin
            if (reset_sync)
                read <= '0;
            else
                read <= (curt_state == STATE_13);
        end

    assign read_done_nxt = (curt_state == STATE_18);

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                read_done <= '0;
            else
                read_done <= read_done_nxt;
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                write <= '0;
            else
                write <= ( (curt_state == STATE_11) &&
                                     (scl_cntnum == 4'd9) );
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                address <= '0;
            else if ( (curt_state == STATE_7) && (scl_cntnum == 4'd8) )
                address <= rxbuf;
            else if ( ((curt_state == STATE_12) && (address != End_Of_ADDR)) ||
                      ((curt_state == STATE_18) && (!i2c_m_ack)) )
                address <= address + 8'd1;
        end

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                writedata <= '0;
            else if ( (curt_state == STATE_10) && (scl_cntnum == 4'd8) )
                writedata <= rxbuf;
        end

    //-----------------------------------------------------------------
    // I2C signals
    //-----------------------------------------------------------------
    assign scl_h2l_trans = (~scl_d1 & scl_d2);
    assign scl_l2h_trans = (scl_d1 & ~scl_d2);
    assign scl_high      = (scl_d1 & scl_d2);

    assign sda_h2l_trans = (~sda_d1 & sda_d2);
    assign sda_l2h_trans = (sda_d1 & ~sda_d2);
    assign sda_high      = (sda_d1 & sda_d2);

    assign i2c_sta = (sda_h2l_trans & scl_high);
    assign i2c_sto = (sda_l2h_trans & scl_high);

    always_ff @(posedge clk)
        begin : I2C_OP_DET
            if (reset_sync)
                i2c_op <= '0;
            else if (curt_state == STATE_4)
                i2c_op <= rxbuf[0];
        end : I2C_OP_DET

    always_ff @(posedge clk)
        begin : I2C_Master_ACK
            if (reset_sync)
                i2c_m_ack <= '1;
            else if ((curt_state == STATE_17) && scl_l2h_trans)
                i2c_m_ack <= sda_i;
        end : I2C_Master_ACK

    always_ff @(posedge clk)
        begin
            if (reset_sync)
                sda_oe_n <= '1;
            else if (curt_state == STATE_16)
                sda_oe_n <= txbuf_shiftout;
            else if ( (curt_state == STATE_5) || (curt_state == STATE_8) ||
                     ((curt_state == STATE_11) && (address != End_Of_ADDR)) )
                sda_oe_n <= '0;
            else
                sda_oe_n <= '1;
        end

    //-----------------------------------------------------------------
    // FSM Statement
    //-----------------------------------------------------------------
    always_ff @(posedge clk)
        begin : FSM_state
            if (reset_sync)
                curt_state <= STATE_1;
            else
                curt_state <= prev_state;
        end : FSM_state

    always_comb begin : FSM_decode
        prev_state = STATE_1;
        case (curt_state)
            STATE_1 : begin
                if (i2c_sta)
                    prev_state = STATE_2;
                else
                    prev_state = STATE_1;
                end
            STATE_2 : begin
                if (scl_h2l_trans)
                    prev_state = STATE_3;
                else
                    prev_state = STATE_2;
                end
            STATE_3 : begin
                if (scl_cntnum < 4'd8)
                    prev_state = STATE_3;
                else
                    prev_state = STATE_4;
                end
            STATE_4 : begin
                if (I2C_DevADDR == rxbuf[7:1])
                    prev_state = STATE_5;
                else
                    prev_state = STATE_19;
                end
            STATE_5 : begin
                if (scl_cntnum < 4'd9)
                    prev_state = STATE_5;
                else
                    prev_state = STATE_6;
                end
            STATE_6 : begin
                if (i2c_op)
                    prev_state = STATE_13;
                else
                    prev_state = STATE_7;
                end
            STATE_7 : begin
                if (i2c_sto)
                    prev_state = STATE_19;
                else if (i2c_sta)
                    prev_state = STATE_2;
                else if (scl_cntnum < 4'd8)
                    prev_state = STATE_7;
                else
                    prev_state = STATE_8;
                end

            STATE_8 : begin
                if (scl_cntnum < 4'd9)
                    prev_state = STATE_8;
                else
                    prev_state = STATE_9;
                end

            STATE_9 : begin
                    prev_state = STATE_10;
                end

            STATE_10 : begin
                if (i2c_sto)
                    prev_state = STATE_19;
                else if (i2c_sta)
                    prev_state = STATE_2;
                else if (scl_cntnum < 4'd8)
                    prev_state = STATE_10;
                else
                    prev_state = STATE_11;
                end

            STATE_11 : begin
                if (scl_cntnum < 4'd9)
                    prev_state = STATE_11;
                else
                    prev_state = STATE_12;
                end

            STATE_12 : begin
                if (address == End_Of_ADDR)
                    prev_state = STATE_19;
                else
                    prev_state = STATE_9;
                end

            STATE_13 : begin
                    prev_state = STATE_14;
                end

            STATE_14 : begin
                    prev_state = STATE_15;
                end

            STATE_15 : begin
                    prev_state = STATE_16;
                end

            STATE_16 : begin
                if (scl_cntnum < 4'd8)
                    prev_state = STATE_16;
                else
                    prev_state = STATE_17;
                end

            STATE_17 : begin
                if (i2c_sto)
                    prev_state = STATE_19;
                else if (i2c_sta)
                    prev_state = STATE_2;
                else if (scl_cntnum < 4'd9)
                    prev_state = STATE_17;
                else
                    prev_state = STATE_18;
                end

            STATE_18 : begin
                if (i2c_m_ack)
                    prev_state = STATE_19;
                else
                    prev_state = STATE_13;
                end

            STATE_19 : begin
                if (scl_high && sda_high)
                    prev_state = STATE_1;
                else
                    prev_state = STATE_19;
                end

             default : begin
                    prev_state = STATE_1;
                end
        endcase
    end : FSM_decode

endmodule
