//---------------------------------------------------------------------
//---------------------------------------------------------------------
// I2C Watchdog 
// Sam Lin, Galaxy TW
// History :
//
// 2014-07-25 : Created.
//
//---------------------------------------------------------------------
`include "../top/es6632bt_iopld_registers.vh"
//---------------------------------------------------------------------
module i2c_wdt (
    input   wire    MGMT_CLK,
    input   wire    MGMT_RST_N,
    input   wire    US_TICK,
    input   wire    HOST_SCL,
    input   wire    HOST_SDA,
    output  wire    SOFT_RST_N
    );

    // The I2C WDT State Encode
    localparam  STATE_1 = 0,
                STATE_2 = 1,
                STATE_3 = 2;

    reg  [1:0]  i2c_wdt_state, i2c_wdt_state_nxt;    

    // The Edge of SCL & SDA detection
    reg             host_scl_high;     
    reg             host_scl_any; 
    wire            host_scl_high_nxt;
    wire            host_scl_any_nxt;    
    
    reg             host_sda_rise;
    reg             host_sda_fall;
    reg             host_sda_high;     
    wire            host_sda_rise_nxt;
    wire            host_sda_fall_nxt;
    wire            host_sda_high_nxt;
    
    reg  [2:0]      host_scl_d;
    reg  [2:0]      host_sda_d;

    wire            sto_cond;
    wire            bus_idle;

    // The Signals for WDT
    reg  [7:0]      iic_wd_cntr;
    reg             iic_wd_cntr_tc;
    
    // The Global Signals
    reg             reset_d1, reset_d2;
    wire            reset_sync;
    
    wire clock   = MGMT_CLK;
    wire reset_n = MGMT_RST_N;

    //-----------------------------------------------------------------
    // Regenerates a sync. reset signal.
    //-----------------------------------------------------------------
    
    assign reset_sync = ~reset_d2;
    
    always @(posedge clock, negedge reset_n) begin
            if (reset_n == 1'b0) begin
                reset_d2 <= 1'b0;
                reset_d1 <= 1'b0;
            end else begin
                reset_d2 <= reset_d1;
                reset_d1 <= 1'b1;
            end
    end
    //-----------------------------------------------------------------
    // Rising & Falling detection for I2C Bus
    //-----------------------------------------------------------------

    assign host_scl_high_nxt   = host_scl_d[2] & host_scl_d[1];
    assign host_scl_any_nxt    = host_scl_d[2] ^ host_scl_d[1];    
    
    assign host_sda_rise_nxt   = (host_sda_d[2:1] == 2'b01) ? 1'b1 : 1'b0;
    assign host_sda_fall_nxt   = (host_sda_d[2:1] == 2'b10) ? 1'b1 : 1'b0;
    assign host_sda_high_nxt   = host_sda_d[2] & host_sda_d[1];
    
    
    always @(posedge clock, posedge reset_sync) begin
            if (reset_sync == 1'b1) begin
                host_scl_d   <= 3'h7;
                host_sda_d   <= 3'h7;
            end else begin
                host_scl_d   <= {host_scl_d[1:0], HOST_SCL};
                host_sda_d   <= {host_sda_d[1:0], HOST_SDA};                
            end
    end
    
    always @(posedge clock, posedge reset_sync) begin
            if (reset_sync == 1'b1) begin
                host_scl_high   <= 1'h0;
                host_scl_any    <= 1'h0;
                host_sda_rise   <= 1'h0;
                host_sda_fall   <= 1'h0;
                host_sda_high   <= 1'h0;
            end else begin
                host_scl_high   <= host_scl_high_nxt;
                host_scl_any    <= host_scl_any_nxt;
                host_sda_rise   <= host_sda_rise_nxt;
                host_sda_fall   <= host_sda_fall_nxt;
                host_sda_high   <= host_sda_high_nxt;
            end
    end
    //-----------------------------------------------------------------
    // I2C Condition statement
    //-----------------------------------------------------------------
    assign sto_cond = (host_sda_rise && host_scl_high) ? 1'b1 : 1'b0;
    assign bus_idle = (host_sda_high && host_scl_high) ? 1'b1 : 1'b0;

    //-----------------------------------------------------------------
    // IIC Watchdog Counter 
    //-----------------------------------------------------------------

    assign SOFT_RST_N = ~iic_wd_cntr_tc;
    
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            {iic_wd_cntr_tc, iic_wd_cntr} <= 9'h000;
        end else if (i2c_wdt_state == STATE_1 ||
                     host_scl_any == 1'b1) begin
                {iic_wd_cntr_tc, iic_wd_cntr} <= 9'h000;
        end else if (i2c_wdt_state == STATE_2 &&
                     US_TICK == 1'b1) begin
                {iic_wd_cntr_tc, iic_wd_cntr} <= iic_wd_cntr + 8'd1;
        end
    end    
    //-----------------------------------------------------------------
    // IIC WDT State Statement
    //-----------------------------------------------------------------
    always @* begin
        // default condition
        i2c_wdt_state_nxt    = STATE_1; 
        case (i2c_wdt_state)
            STATE_1 : begin
                if (host_sda_fall == 1'b1) begin
                    i2c_wdt_state_nxt       = STATE_2;
                end else begin
                    i2c_wdt_state_nxt       = STATE_1;
                end
            end
            STATE_2 : begin
                if (iic_wd_cntr_tc == 1'b1) begin
                    i2c_wdt_state_nxt       = STATE_3;
                end else if (sto_cond == 1'b1) begin
                    i2c_wdt_state_nxt       = STATE_1;
                end else begin
                    i2c_wdt_state_nxt       = STATE_2;
                end
            end
            STATE_3 : begin
                if (bus_idle == 1'b1) begin
                    i2c_wdt_state_nxt       = STATE_1;
                end else begin
                    i2c_wdt_state_nxt       = STATE_3;
                end
            end            
            default : begin
                    i2c_wdt_state_nxt       = STATE_1;
            end
        endcase
    end

    // Flops Inference
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            i2c_wdt_state          <= STATE_1;
        end else begin
            i2c_wdt_state          <= i2c_wdt_state_nxt;
        end    
    end
    
endmodule
