//*********************************************************************
// This is a FAN controler with a temperature chip (LM75)
// 2014-08-26
// sam.lin@gfec.com.tw, GALAXY TW
// History :
// 08-26-14  Created
//
//*********************************************************************
`include "./es7632bt_fanpld_defined.vh"

module temper_ctrl (
    input            MGMT_CLK,
    input            MGMT_RST_N,
    input            MASK_I2C_LM75,
	input            I2C_MASTER_ENB,
    input            FAN_IDEL,
    input            SCL,
    input            HALF_SEC_TICK,    // ~0.5 sec
    // I2C Core
    output reg       I2C_START,
    output reg       I2C_STOP,
    output reg       I2C_READ,
    output reg       I2C_WRITE,
    output reg       I2C_ACK_SEND,
    output reg [7:0] I2C_WDATA,
    input            I2C_CMD_ACK,
    input            I2C_ACK_RECV,
    input      [7:0] I2C_RDATA,
    // FAN
    output reg       FAN_STOP_INT,
    output reg       FAN_PWM_UPDATA,
    output     [3:0] FAN_PWM_CTRL
    );

    // FSM Encode for The I2C Master Core
    localparam  stm_WAIT = 0,
                stm_I2C_WR_CMD = 1,
                stm_I2C_WR_CMD_ACK = 2,
                stm_I2C_WR_REG_ADDR = 3,
                stm_I2C_WR_REG_ADDR_ACK = 4,
                stm_I2C_RD_CMD = 5,
                stm_I2C_RD_CMD_ACK = 6,
                stm_I2C_RD_FIRST_BYTE = 7,
                stm_I2C_RD_FIRST_BYTE_ACK = 8,
                //stm_I2C_RD_LAST_BYTE = 5,
                //stm_I2C_RD_LAST_BYTE_ACK = 6,
                stm_STOP = 9;

    reg [3:0]   i2c_m_op_state, i2c_m_op_state_nxt;

    // Signals for I2C Master
    reg         busy_nxt, busy;
    reg         i2c_start_nxt;
    reg         i2c_stop_nxt;
    reg         i2c_read_nxt;
    reg         i2c_write_nxt;
    reg  [7:0]  i2c_wdata_nxt;
    reg  [7:0]  i2c_recvdata_tmp;
    reg  [7:0]  i2c_senddata_tmp;
    reg  [7:0]  i2c_addr_byte_tmp;
    reg         i2c_master_ack_nxt;
    reg  [1:0]  scl_d;
    reg         i2c_core_cmd_ack_sig;
    reg         i2c_core_slave_ack_sig;
    wire        scl_rise_sig;
    wire        scl_fall_sig;
    reg  [2:0]  two_second_cntr;
    wire        sec_two_tick_nxt;
    reg         sec_two_tick;
    reg         sec_half_tick;
    // Signals for FAN Control
    reg  [1:0]  pointer_cntr;

    reg  [3:0]  r_fan_pwm_ctrl_4b;
    reg         fan_pwm_updata_nxt;
    reg         fan_stop_int_nxt;

    // Signals for LM75
    reg  [7:0] lm75_temper_data;
    reg  [7:0] lm75_1_temper_data;
    reg  [7:0] lm75_2_temper_data;
    reg  [7:0] lm75_3_temper_data;

    // Clocks Signals
    reg         reset_d1 = 1'b0;
    reg         reset_d2 = 1'b0;
    wire        reset_sync;
    reg         fan_idle_ff;

    // Buffers
    wire   clock   = MGMT_CLK;
    wire   reset_n = MGMT_RST_N;
    assign FAN_PWM_CTRL   = r_fan_pwm_ctrl_4b;

    always @(posedge clock) begin
        i2c_core_cmd_ack_sig   <= I2C_CMD_ACK;
        i2c_core_slave_ack_sig <= I2C_ACK_RECV;
        sec_half_tick          <= HALF_SEC_TICK;
        i2c_recvdata_tmp       <= I2C_RDATA;
        sec_two_tick           <= sec_two_tick_nxt;
        fan_idle_ff            <= FAN_IDEL;
    end

    //-----------------------------------------------------------------
    // Regenerates a sync. reset signal.
    //-----------------------------------------------------------------
    assign reset_sync = ~reset_d2;

    always @(posedge clock, negedge reset_n) begin
            if (reset_n == 1'b0) begin
                reset_d2 <= 1'b0;
                reset_d1 <= 1'b0;
            end else begin
                reset_d2 <= reset_d1;
                reset_d1 <= 1'b1;
            end
    end

    //-----------------------------------------------------------------
    // The edge detection
    //-----------------------------------------------------------------
    always @(posedge clock, posedge reset_sync) begin
            if (reset_sync == 1'b1) begin
                scl_d   <= 2'h3;
            end else begin
                scl_d   <= {scl_d[0], SCL};
            end
    end

    assign scl_rise_sig   = (scl_d == 2'b01) ? 1'b1 : 1'b0;
    assign scl_fall_sig   = (scl_d == 2'b10) ? 1'b1 : 1'b0;

    //-----------------------------------------------------------------
    // To get the temperature data form LM75
    //-----------------------------------------------------------------
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            lm75_1_temper_data <= 8'h00;
            lm75_2_temper_data <= 8'h00;
            lm75_3_temper_data <= 8'h00;
        end else begin
            if (i2c_m_op_state_nxt == stm_I2C_RD_FIRST_BYTE_ACK &&
                pointer_cntr == 2'd0)
                lm75_1_temper_data <= i2c_recvdata_tmp;
            else if (i2c_m_op_state_nxt == stm_I2C_RD_FIRST_BYTE_ACK &&
                pointer_cntr == 2'd1)
                lm75_2_temper_data <= i2c_recvdata_tmp;
            else if (i2c_m_op_state_nxt == stm_I2C_RD_FIRST_BYTE_ACK &&
                pointer_cntr == 2'd2)
                lm75_3_temper_data <= i2c_recvdata_tmp;
        end
    end

    // To compare highest tempuration with the LM75 devices
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            lm75_temper_data <= 8'h00;
        end else begin
            if ((lm75_1_temper_data >= lm75_2_temper_data) &&
                (lm75_1_temper_data >= lm75_3_temper_data))
                lm75_temper_data <= lm75_1_temper_data;
            else if ((lm75_2_temper_data >= lm75_1_temper_data) &&
                (lm75_2_temper_data >= lm75_3_temper_data))
                lm75_temper_data <= lm75_2_temper_data;
            else if ((lm75_3_temper_data >= lm75_1_temper_data) &&
                (lm75_3_temper_data >= lm75_2_temper_data))
                lm75_temper_data <= lm75_3_temper_data;
        end
    end

    //-----------------------------------------------------------------
    // Fan PWM Control Logic
    //-----------------------------------------------------------------
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            r_fan_pwm_ctrl_4b <= 4'h5;
        end else begin
            if ((lm75_temper_data >= `TEMPERATURE_30 &&
                lm75_temper_data < `TEMPERATURE_40)) begin

                r_fan_pwm_ctrl_4b <= 4'h6;

            end if (lm75_temper_data == `TEMPERATURE_40) begin

                r_fan_pwm_ctrl_4b <= 4'h7;

            end if (lm75_temper_data == `TEMPERATURE_41) begin

                r_fan_pwm_ctrl_4b <= 4'h8;

            end if (lm75_temper_data == `TEMPERATURE_42) begin

                r_fan_pwm_ctrl_4b <= 4'h9;

            end if (lm75_temper_data == `TEMPERATURE_43) begin

                r_fan_pwm_ctrl_4b <= 4'hA;

            end if (lm75_temper_data == `TEMPERATURE_44) begin

                r_fan_pwm_ctrl_4b <= 4'hB;

            end if (lm75_temper_data == `TEMPERATURE_45) begin

                r_fan_pwm_ctrl_4b <= 4'hC;

            end if (lm75_temper_data == `TEMPERATURE_46) begin

                r_fan_pwm_ctrl_4b <= 4'hD;

            end if ((lm75_temper_data >= `TEMPERATURE_47 &&
                lm75_temper_data < `TEMPERATURE_50)) begin

                r_fan_pwm_ctrl_4b <= 4'hE;

            end if (lm75_temper_data >= `TEMPERATURE_50) begin

                r_fan_pwm_ctrl_4b <= 4'hF;

            end else begin
                r_fan_pwm_ctrl_4b <= 4'h5;
            end
        end
    end

    //-----------------------------------------------------------------
    // The Pointer Counter for Read LM75 devices
    //-----------------------------------------------------------------
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            pointer_cntr <= 2'h0;
        end else begin
            if (i2c_m_op_state_nxt == stm_STOP && pointer_cntr == 2'd2)
                pointer_cntr <= 2'h0;
            else if (i2c_m_op_state_nxt == stm_STOP)
                pointer_cntr <= pointer_cntr + 2'd1;
        end
    end

    // Time Tick
    assign sec_two_tick_nxt = (two_second_cntr == 3'd4) ? 1'b1 : 1'b0;

    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            two_second_cntr <= 3'd0;
        end else begin
            if (two_second_cntr == 3'd4) begin
                two_second_cntr <= 3'd0;
            end else if (sec_half_tick == 1'b1) begin
                two_second_cntr <= two_second_cntr + 3'd1;
            end
        end
    end

    //-----------------------------------------------------------------
    // The Write Data of I2C Master Core Statement
    //-----------------------------------------------------------------
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            i2c_senddata_tmp  <= 8'h00;
            i2c_addr_byte_tmp <= 8'h00;
        end else begin
            if (pointer_cntr == 2'd0) begin
                i2c_senddata_tmp  <= `RD_LM75_1;
                i2c_addr_byte_tmp <= `WR_LM75_1;
            end
            if (pointer_cntr == 2'd1) begin
                i2c_senddata_tmp <=  `RD_LM75_2;
                i2c_addr_byte_tmp <= `WR_LM75_2;
            end
            if (pointer_cntr == 2'd2) begin
                i2c_senddata_tmp  <= `RD_LM75_3;
                i2c_addr_byte_tmp <= `WR_LM75_3;
            end
        end
    end

    //-----------------------------------------------------------------
    // FSM Statement of I2C Mater Core
    //-----------------------------------------------------------------

    // FSM Decode
    always @* begin
        // default condition
        i2c_m_op_state_nxt  = stm_WAIT;
        i2c_start_nxt       = 1'b0;
        i2c_stop_nxt        = 1'b0;
        i2c_read_nxt        = 1'b0;
        i2c_write_nxt       = 1'b0;
        i2c_master_ack_nxt  = 1'b0;
        fan_pwm_updata_nxt  = 1'b0;
        fan_stop_int_nxt    = 1'b0;
        i2c_wdata_nxt       = i2c_addr_byte_tmp;

        case (i2c_m_op_state)
            stm_WAIT : begin
                // Waiting Time that is about half second.
                if (sec_half_tick == 1'b1 && fan_idle_ff == 1'b0 && MASK_I2C_LM75 == 1'b1) begin
//                if (sec_half_tick == 1'b1) begin // for simulation
                    i2c_m_op_state_nxt      = stm_I2C_WR_CMD;
                    i2c_start_nxt           = 1'b1;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b1;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_addr_byte_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_WAIT;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_addr_byte_tmp;
                end
            end
            stm_I2C_WR_CMD : begin
                if (i2c_core_cmd_ack_sig == 1'b1) begin // Waiting for the acknowledgement on the I2C Core
                                                        // during the device address
                    i2c_m_op_state_nxt      = stm_I2C_WR_CMD_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_addr_byte_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_WR_CMD;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_addr_byte_tmp;
                end
            end
            stm_I2C_WR_CMD_ACK : begin
                if (fan_idle_ff == 1'b1) begin
                    i2c_m_op_state_nxt      = stm_STOP;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b1;
                    i2c_wdata_nxt           = 8'h00;

                end else if (scl_fall_sig == 1'b1 &&
                    i2c_core_slave_ack_sig == 1'b0) begin
                    // I2C Slave ACK from the device.
                    i2c_m_op_state_nxt      = stm_I2C_WR_REG_ADDR;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b1;
                    i2c_master_ack_nxt      = 1'b0; // 1'b1;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00; // To send the address byte of the register
                end else if (scl_fall_sig == 1'b1 &&
                    i2c_core_slave_ack_sig == 1'b1) begin
                    // I2C Slave is no ACK.
                    i2c_m_op_state_nxt      = stm_STOP;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_WR_CMD_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_addr_byte_tmp;
                end
            end
            stm_I2C_WR_REG_ADDR : begin
                if (i2c_core_cmd_ack_sig == 1'b1) begin // Waiting for the acknowledgement on the I2C Core
                                                        // during the device address
                    i2c_m_op_state_nxt      = stm_I2C_WR_REG_ADDR_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00; // To send the address byte of the register
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_WR_REG_ADDR;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00; // To send the address byte of the register
                end
            end
            stm_I2C_WR_REG_ADDR_ACK : begin
                if (fan_idle_ff == 1'b1) begin
                    i2c_m_op_state_nxt      = stm_STOP;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b1;
                    i2c_wdata_nxt           = 8'h00;

                end else if (scl_fall_sig == 1'b1 &&
                    i2c_core_slave_ack_sig == 1'b0) begin
                    // I2C Slave ACK from the device.
                    i2c_m_op_state_nxt      = stm_I2C_RD_CMD;
                    i2c_start_nxt           = 1'b1; // re-start
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b1;
                    i2c_master_ack_nxt      = 1'b0; // 1'b1;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else if (scl_fall_sig == 1'b1 &&
                    i2c_core_slave_ack_sig == 1'b1) begin
                    // I2C Slave is no ACK.
                    i2c_m_op_state_nxt      = stm_STOP;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_WR_REG_ADDR_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            stm_I2C_RD_CMD : begin
                if (i2c_core_cmd_ack_sig == 1'b1) begin // Waiting for the acknowledgement on the I2C Core
                                                        // during the device address
                    i2c_m_op_state_nxt      = stm_I2C_RD_CMD_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_RD_CMD;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0; // 1'b1;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            stm_I2C_RD_CMD_ACK : begin
                if (fan_idle_ff == 1'b1) begin
                    i2c_m_op_state_nxt      = stm_STOP;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b1;
                    i2c_wdata_nxt           = i2c_senddata_tmp;

                end else if (scl_fall_sig == 1'b1 &&
                    i2c_core_slave_ack_sig == 1'b0) begin
                    // I2C Slave ACK from the device.
                    i2c_m_op_state_nxt      = stm_I2C_RD_FIRST_BYTE;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b1;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0; // 1'b1;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else if (scl_fall_sig == 1'b1 &&
                    i2c_core_slave_ack_sig == 1'b1) begin
                    // I2C Slave is no ACK.
                    i2c_m_op_state_nxt      = stm_STOP;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_RD_CMD_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            stm_I2C_RD_FIRST_BYTE : begin
                if (i2c_core_cmd_ack_sig == 1'b1) begin // Waiting for the acknowledgement
                                                        // during reading first byte.
                    i2c_m_op_state_nxt      = stm_I2C_RD_FIRST_BYTE_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0; // 1'b1;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_RD_FIRST_BYTE;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b1; // To acknowledge by the master.
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            stm_I2C_RD_FIRST_BYTE_ACK : begin
                if (scl_fall_sig == 1'b1 || fan_idle_ff == 1'b1) begin
                    i2c_m_op_state_nxt      = stm_STOP; // Returning to first state from stop I2C Core.
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b1;
                    fan_stop_int_nxt        = (FAN_IDEL == 1'b1) ? 1'b1 : 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_RD_FIRST_BYTE_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            /*
            stm_I2C_RD_LAST_BYTE : begin
                if (i2c_core_cmd_ack_sig == 1'b1) begin // Waiting for the acknowledgement
                                                        // during reading last byte.
                    i2c_m_op_state_nxt      = stm_I2C_RD_LAST_BYTE_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_RD_LAST_BYTE;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b1;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            stm_I2C_RD_LAST_BYTE_ACK : begin
                if (scl_fall_sig == 1'b1) begin
                    i2c_m_op_state_nxt      = stm_STOP; // Returning to first state from stop I2C Core.
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b1;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end else begin
                    i2c_m_op_state_nxt      = stm_I2C_RD_LAST_BYTE_ACK;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    i2c_wdata_nxt           = i2c_senddata_tmp;
                end
            end
            */
            stm_STOP : begin
                    i2c_m_op_state_nxt      = stm_WAIT;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00;
            end
            default : begin
                    i2c_m_op_state_nxt      = stm_WAIT;
                    i2c_start_nxt           = 1'b0;
                    i2c_stop_nxt            = 1'b0;
                    i2c_read_nxt            = 1'b0;
                    i2c_write_nxt           = 1'b0;
                    i2c_master_ack_nxt      = 1'b0;
                    fan_pwm_updata_nxt      = 1'b0;
                    fan_stop_int_nxt        = 1'b0;
                    i2c_wdata_nxt           = 8'h00;
            end
        endcase
    end

    // Flops Inference
    always @(posedge clock, posedge reset_sync) begin
        if (reset_sync == 1'b1) begin
            i2c_m_op_state      <= stm_WAIT;
            I2C_START           <= 1'b0;
            I2C_STOP            <= 1'b0;
            I2C_READ            <= 1'b0;
            I2C_WRITE           <= 1'b0;
            I2C_ACK_SEND        <= 1'b0;
            I2C_WDATA           <= 8'h00;
            FAN_PWM_UPDATA      <= 1'b0;
            FAN_STOP_INT        <= 1'b0;
        end else if (I2C_MASTER_ENB == 1'b1) begin
            i2c_m_op_state      <= i2c_m_op_state_nxt;
            I2C_START           <= i2c_start_nxt;
            I2C_STOP            <= i2c_stop_nxt;
            I2C_READ            <= i2c_read_nxt;
            I2C_WRITE           <= i2c_write_nxt;
            I2C_ACK_SEND        <= i2c_master_ack_nxt;
            I2C_WDATA           <= i2c_wdata_nxt;
            FAN_PWM_UPDATA      <= fan_pwm_updata_nxt;
            FAN_STOP_INT        <= fan_stop_int_nxt;
        end
    end

endmodule
