//----------------------------------------------------------------------
// Watchdog for FAN CPLD
// Sam Lin, Galaxy TW
// History :
//
// 2015-02-02 Created
`include "../top/es7632bt_fanpld_defined.vh"
//----------------------------------------------------------------------
module watchdog (
    input           MGMT_CLK,
    input           MGMT_RST_N,
    input           I2C_WRITE,
    input   [7:0]   I2C_OFFSET_ADDR,
    input   [7:0]   I2C_OFFSER_WDATA,
    output          WD_PWM_CTRL,
    output  [7:0]   REG_WD_TIMER_8B,
    output  [3:0]   REG_WD_PWM_MAX_VALUE_4B,
    output          REG_WD_ENABLE_1B
);

    // Edge detection for any signals
    wire        reset_n;
    wire        clock;
    reg  [24:0] clock_1hz_cntr = 25'h0000000;
    reg  [7:0]  r_wd_counter_8b = 8'h06;
    reg  [3:0]  r_wdt_pwm_ctrl_4b = 4'hF;
    reg         r_wdt_enabale_1b = 1'b1;
    reg         r_wdt_pwm_ctrl;

    // Buffers
    assign  clock   = MGMT_CLK;
    assign  reset_n = MGMT_RST_N;


    // Generates a tick to count watchdog timer
    always @(posedge clock) begin
        if (clock_1hz_cntr == `ONE_SECOND) begin
            clock_1hz_cntr <= 25'd0;
        end else begin
            clock_1hz_cntr <= clock_1hz_cntr + 25'd1;
        end
    end

    //---------------------------------------------------------------------
    // Watchdog Timer Register -- 0x31
    //---------------------------------------------------------------------
    assign REG_WD_TIMER_8B = r_wd_counter_8b;
    assign WD_PWM_CTRL     = r_wdt_pwm_ctrl;

    always @(posedge clock) begin
       if (reset_n == 1'b0 ) begin
            r_wd_counter_8b <= `WDT_DEFAULT;
            r_wdt_pwm_ctrl  <= 1'h0;
       end else begin
       		if (r_wdt_enabale_1b == 1'b0) begin
       			r_wd_counter_8b <= `WDT_DEFAULT; // To avoid underflow of the counter
                r_wdt_pwm_ctrl  <= 1'b0;
            end else if (r_wd_counter_8b == 8'd0) begin
                r_wd_counter_8b <= `WDT_DEFAULT; // To avoid underflow of the counter
                r_wdt_pwm_ctrl  <= 1'b1;
            end else if (I2C_WRITE == 1'b1 &&
                         I2C_OFFSET_ADDR == `FANPLD_REG_FAN_WD_TIMER) begin
                r_wd_counter_8b <= I2C_OFFSER_WDATA;
                r_wdt_pwm_ctrl  <= 1'b0;
            end else if (r_wdt_enabale_1b == 1'b1 && clock_1hz_cntr == `ONE_SECOND) begin
                r_wd_counter_8b <= r_wd_counter_8b - 8'd1;
                r_wdt_pwm_ctrl  <= 1'b0;
            end
       end
    end

    //---------------------------------------------------------------------
    // Watchdog Maximum PWM Value Register -- 0x32
    //---------------------------------------------------------------------
    assign REG_WD_PWM_MAX_VALUE_4B = r_wdt_pwm_ctrl_4b;

    always @(posedge clock) begin
       if (reset_n == 1'b0 ) begin
            r_wdt_pwm_ctrl_4b      <= 4'b1111;
       end else if (I2C_WRITE == 1'b1 &&
                    I2C_OFFSET_ADDR == `FANPLD_REG_FAN_WD_PWM_CTRL ) begin
            r_wdt_pwm_ctrl_4b <= I2C_OFFSER_WDATA[3:0];
       end
    end

    //---------------------------------------------------------------------
    // Watchdog Disable Register -- 0x33
    //---------------------------------------------------------------------
    assign REG_WD_ENABLE_1B = r_wdt_enabale_1b;

    always @(posedge clock) begin
       if (reset_n == 1'b0 ) begin
            r_wdt_enabale_1b <= 1'b1;
    end else if (I2C_WRITE == 1'b1 && I2C_OFFSET_ADDR == `FANPLD_REG_FAN_WD_DISABLE ) begin
            r_wdt_enabale_1b <= I2C_OFFSER_WDATA[0];
       end
    end

endmodule
